@@ -6,24 +6,23 @@
 // TODO: work to remove the following lints
 #![allow(clippy::missing_errors_doc)]
 
+pub mod common;
 pub mod config;
-pub mod ids;
 pub mod invidious;
-pub mod library_db;
+pub mod new_database;
 pub mod player;
 pub mod playlist;
 pub mod podcast;
 pub mod songtag;
 pub mod taskpool;
 pub mod track;
-pub mod types;
 pub mod ueberzug;
 pub mod utils;
 pub mod xywh;
 
 pub const VERSION: &str = env!("CARGO_PKG_VERSION");
 
-use include_dir::{include_dir, Dir};
+use include_dir::{Dir, include_dir};
 pub static THEME_DIR: Dir<'_> = include_dir!("$CARGO_MANIFEST_DIR/themes");
 
 #[macro_use]
