@@ -11,7 +11,7 @@ use std::{
     time::{Duration, SystemTime},
 };
 
-use anyhow::{anyhow, bail, Context, Result};
+use anyhow::{Context, Result, anyhow, bail};
 use id3::frame::Lyrics as Id3Lyrics;
 use lofty::{
     config::ParseOptions,
@@ -384,10 +384,10 @@ impl Track {
                 PlaylistTrackSource::Path(track_data.path.to_string_lossy().to_string())
             }
             MediaTypes::Radio(radio_track_data) => {
-                PlaylistTrackSource::Url(radio_track_data.url.to_string())
+                PlaylistTrackSource::Url(radio_track_data.url.clone())
             }
             MediaTypes::Podcast(podcast_track_data) => {
-                PlaylistTrackSource::PodcastUrl(podcast_track_data.url.to_string())
+                PlaylistTrackSource::PodcastUrl(podcast_track_data.url.clone())
             }
         }
     }
@@ -541,6 +541,11 @@ fn get_picture_for_music_track(track_path: &Path) -> Result<Option<Picture>> {
     Ok(Some(picture))
 }
 
+/// All extension we support to find in a parent folder of a given track to use as a cover image.
+///
+/// The matching is done via lowercase EQ.
+const SUPPORTED_IMG_EXTENSIONS: &[&str] = &["jpg", "jpeg", "png"];
+
 /// Find a picture file and parse it in the parent directory of the given path.
 ///
 /// # Errors
@@ -569,7 +574,10 @@ fn find_folder_picture(track_path: &Path) -> Result<Option<Picture>> {
         };
 
         // only take some picture files we can handle and are common
-        if ext != "jpg" || ext != "png" {
+        if !SUPPORTED_IMG_EXTENSIONS
+            .iter()
+            .any(|v| ext.eq_ignore_ascii_case(v))
+        {
             continue;
         }
 
@@ -616,10 +624,6 @@ impl Display for DurationFmtShort {
     }
 }
 
-/// The default and most common separators used for artists.
-pub const DEFAULT_ARTIST_SEPARATORS: &[&str] =
-    &[",", ";", "&", "ft.", "feat.", "/", "|", "×", "、", " x "];
-
 /// See [`TrackMetadata`] for explanation of values.
 #[derive(Debug, Clone, Copy, PartialEq, Default)]
 #[allow(clippy::struct_excessive_bools)] // configuration, this is not a state machine
@@ -780,6 +784,8 @@ fn handle_tag(tag: &LoftyTag, options: MetadataOptions<'_>, res: &mut TrackMetad
             .map(ToString::to_string);
     }
     if options.album_artists {
+        // TODO: manually split if convenient tag is not available
+
         // manual implementation as it currently does not exist upstream
         // see https://github.com/Serial-ATA/lofty-rs/issues/522
         // res.album_artists = Some(tag.get_strings(&ItemKey::AlbumArtists).map(ToString::to_string).collect());
@@ -844,7 +850,7 @@ fn get_lyrics_from_tags(tag: &LoftyTag, lyric_frames: &mut Vec<Id3Lyrics>) {
             lyric_frames.push(Id3Lyrics {
                 lang: lyric.lang().escape_ascii().to_string(),
                 description: lyric.description().to_string(),
-                text: lyrics_text.to_string(),
+                text: lyrics_text.clone(),
             });
         }
     }
