@@ -1,23 +1,23 @@
 use std::fmt::Display;
 use std::fs::File;
 use std::path::Path;
+use std::sync::Arc;
 use std::sync::atomic::{AtomicU16, Ordering};
 use std::sync::mpsc::{self, Receiver, Sender};
-use std::sync::Arc;
 use std::time::Duration;
 
 use anyhow::{Context, Result};
 use async_trait::async_trait;
 use parking_lot::Mutex;
-use rodio::OutputStream;
+use rodio::OutputStreamBuilder;
 use rodio::Source;
 use std::num::{NonZeroU16, NonZeroUsize};
 use stream_download::http::{
-    reqwest::{
-        header::{HeaderMap, HeaderValue},
-        Client,
-    },
     HttpStream,
+    reqwest::{
+        Client,
+        header::{HeaderMap, HeaderValue},
+    },
 };
 use stream_download::source::SourceStream;
 use stream_download::storage::bounded::BoundedStorageProvider;
@@ -88,7 +88,9 @@ impl RustyBackend {
         let volume_local = volume.clone();
         let speed = config_read.settings.player.speed;
         let gapless = config_read.settings.player.gapless;
+        let output_sample_rate = config_read.settings.backends.rusty.output_sample_rate;
         drop(config_read);
+
         let position = Arc::new(Mutex::new(Duration::default()));
         let total_duration = Arc::new(Mutex::new(None));
         let total_duration_local = total_duration.clone();
@@ -114,6 +116,7 @@ impl RustyBackend {
                     position: position_local,
                     volume_inside: volume_local,
                     speed_inside: speed,
+                    output_sample_rate,
                 }));
             })
             .expect("failed to spawn thread");
@@ -342,18 +345,19 @@ fn append_to_sink_inner<TD: Display, F: FnOnce(&mut Symphonia, Option<MediaTitle
 
     if common_options.async_decode {
         let handle = tokio::runtime::Handle::current();
-        let (spec, current_frame_len) = decoder.get_spec();
+        let (spec, current_span_len) = decoder.get_spec();
         let total_duration = decoder.total_duration();
         let (prod, cons) = AsyncRingSource::new(
             spec,
             total_duration,
-            current_frame_len,
+            current_span_len,
             common_options.ringbuf_size,
             handle.clone(),
         );
 
         tokio::task::spawn_blocking(move || {
             handle.block_on(decode_task(decoder, prod));
+            trace!("Decode task returned");
         });
 
         sink.append(
@@ -376,6 +380,7 @@ fn append_to_sink_inner<TD: Display, F: FnOnce(&mut Symphonia, Option<MediaTitle
 
 /// The task that runs the decoder and writes to the ringbuffer, until a error or the consumer closes.
 async fn decode_task(mut decoder: Symphonia, mut prod: AsyncRingSourceProvider) -> Option<()> {
+    let mut send_eos = false;
     loop {
         // will always write the full buffer as long as the consumer is connected
         let seek_fut = prod.wait_seek();
@@ -389,11 +394,18 @@ async fn decode_task(mut decoder: Symphonia, mut prod: AsyncRingSourceProvider)
 
         select! {
             // if there is nothing to write, this future may exit immediately, causing a fast-loop until seek or dropped.
-            written = write_fut, if !exhausted_buffer => {
-                written.ok()?;
-                decoder.advance_offset(decoder.get_buffer().len());
+            // that is why condition is here, to only listen on the "seek" future.
+            written = write_fut, if !send_eos => {
+                // only check the result and advance the buffer if the send buffer is not 0-length
+                // as "written" in that case will be "Err".
+                if !exhausted_buffer {
+                    written.ok()?;
+                    decoder.advance_offset(decoder.get_buffer().len());
+                }
             },
             seek = seek_fut => {
+                send_eos = false;
+                // "seek" being "None" here means that the seek channel is closed, which in turn means we can abort decoding.
                 let seek = seek?;
                 decode_task_seek_fut(&mut decoder, &mut prod, seek).await?;
             }
@@ -401,8 +413,9 @@ async fn decode_task(mut decoder: Symphonia, mut prod: AsyncRingSourceProvider)
 
         let spec_len = decoder.get_spec();
         if decoder.decode_once().is_none() {
-            trace!("Sending EOS");
+            trace!("Sending decoder EOS");
             prod.new_eos().await.ok()?;
+            send_eos = true;
         }
         let new_spec = decoder.get_spec();
         if spec_len != new_spec {
@@ -570,6 +583,8 @@ struct PlayerThreadArgs {
 
     volume_inside: Arc<AtomicU16>,
     speed_inside: i32,
+
+    output_sample_rate: u32,
 }
 
 /// Player thread loop
@@ -585,8 +600,19 @@ async fn player_thread(mut args: PlayerThreadArgs) {
     // option to store enqueued's duration
     // note that the current implementation is only meant to have 1 enqueued next after the current playing song
     let mut next_duration_opt = None;
-    let (_stream, handle) = OutputStream::try_default().unwrap();
-    let sink = Sink::try_new(&handle, args.picmd_tx.clone(), args.pcmd_tx.clone()).unwrap();
+    // Tracks whether a "About to Finish" message had already been send or not, to not spam the messages.
+    // This needs to be reset on many occasions like Seek or Stream Start.
+    let mut send_atf = false;
+
+    let stream = {
+        let builder = OutputStreamBuilder::from_default_device().unwrap();
+        builder
+            .with_sample_rate(args.output_sample_rate)
+            .open_stream_or_fallback()
+            .unwrap()
+    };
+    let handle = stream.mixer();
+    let sink = Sink::try_new(handle, args.picmd_tx.clone(), args.pcmd_tx.clone());
     sink.set_speed(args.speed_inside as f32 / 10.0);
     sink.set_volume(f32::from(args.volume_inside.load(Ordering::SeqCst)) / 100.0);
     loop {
@@ -656,8 +682,8 @@ async fn player_thread(mut args: PlayerThreadArgs) {
                 // error!("position in rusty backend is: {}", position);
                 *args.position.lock() = new_position;
 
-                // About to finish signal is a simulation of gstreamer, and used for gapless
-                if !is_radio {
+                // Send a "About to Finish" signal to start pre-fetching / enqueue the next track
+                if !is_radio && !send_atf {
                     if let Some(d) = *args.total_duration.lock() {
                         let progress = new_position.as_secs_f64() / d.as_secs_f64();
                         if progress >= 0.5
@@ -666,15 +692,18 @@ async fn player_thread(mut args: PlayerThreadArgs) {
                             if let Err(e) = args.pcmd_tx.send(PlayerCmd::AboutToFinish) {
                                 error!("command AboutToFinish sent failed: {e}");
                             }
+                            send_atf = true;
                         }
                     }
                 }
             }
             PlayerInternalCmd::SeekAbsolute(position) => {
+                send_atf = false;
                 sink.seek(position);
             }
 
             PlayerInternalCmd::SeekRelative(offset) => {
+                send_atf = false;
                 let paused = sink.is_paused();
                 if paused {
                     sink.set_volume(0.0);
@@ -701,6 +730,7 @@ async fn player_thread(mut args: PlayerThreadArgs) {
             }
 
             PlayerInternalCmd::Eos => {
+                send_atf = false;
                 // replace the current total_duration with the next one
                 // this is only present when QueueNext was used; which is only used if gapless is enabled
                 if next_duration_opt.is_some() {
@@ -737,6 +767,14 @@ async fn queue_next(
     next_duration_opt: &mut Option<Duration>,
     media_title: &Arc<Mutex<String>>,
 ) -> Result<()> {
+    // clear out the sources when we dont "enqueue" as we want to directly play it
+    if !options.enqueue && !sink.is_empty() {
+        // dont have the source that are cleared-out send a EOS as that the player-trait does not know about those sources
+        // and assumes the one that is about to be added has finished.
+        sink.stop_no_eos();
+        // not waiting here as the sink automatically does that on ".append"
+    }
+
     match track.inner() {
         MediaTypes::Track(track_data) => {
             *is_radio = false;
