@@ -5,23 +5,23 @@ use std::io::{BufRead, BufReader, BufWriter, Write};
 use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
-use anyhow::{bail, Context, Result};
+use anyhow::{Context, Result, bail};
 use parking_lot::RwLock;
 use pathdiff::diff_paths;
-use rand::seq::SliceRandom;
 use rand::Rng;
-use termusiclib::config::v2::server::LoopMode;
+use rand::seq::SliceRandom;
 use termusiclib::config::SharedServerSettings;
-use termusiclib::player::playlist_helpers::PlaylistPlaySpecific;
-use termusiclib::player::playlist_helpers::PlaylistSwapTrack;
-use termusiclib::player::playlist_helpers::PlaylistTrackSource;
-use termusiclib::player::playlist_helpers::{PlaylistAddTrack, PlaylistRemoveTrackIndexed};
+use termusiclib::config::v2::server::LoopMode;
 use termusiclib::player::PlaylistLoopModeInfo;
 use termusiclib::player::PlaylistShuffledInfo;
 use termusiclib::player::PlaylistSwapInfo;
 use termusiclib::player::PlaylistTracks;
 use termusiclib::player::UpdateEvents;
 use termusiclib::player::UpdatePlaylistEvents;
+use termusiclib::player::playlist_helpers::PlaylistPlaySpecific;
+use termusiclib::player::playlist_helpers::PlaylistSwapTrack;
+use termusiclib::player::playlist_helpers::PlaylistTrackSource;
+use termusiclib::player::playlist_helpers::{PlaylistAddTrack, PlaylistRemoveTrackIndexed};
 use termusiclib::player::{self, RunningStatus};
 use termusiclib::player::{PlaylistAddTrackInfo, PlaylistRemoveTrackInfo};
 use termusiclib::podcast::{db::Database as DBPod, episode::Episode};
@@ -37,7 +37,9 @@ pub struct Playlist {
     tracks: Vec<Track>,
     /// Index into `tracks` of which the current playing track is
     current_track_index: usize,
-    /// Index into `tracks` for the next track to play after the current
+    /// Index into `tracks` for the next track to play after the current.
+    ///
+    /// Practically only used for pre-enqueue / pre-fetch / gapless.
     next_track_index: Option<usize>,
     /// The currently playing [`Track`]. Does not need to be in `tracks`
     current_track: Option<Track>,
@@ -424,6 +426,9 @@ impl Playlist {
     ///
     /// This uses `played_index` vec, if available, otherwise uses [`LoopMode`].
     pub fn previous(&mut self) {
+        // unset next track as we now want a previous track instead of the next enqueued
+        self.set_next_track(None);
+
         if !self.played_index.is_empty() {
             if let Some(index) = self.played_index.pop() {
                 self.current_track_index = index;
@@ -505,7 +510,7 @@ impl Playlist {
         let index_a = u64::try_from(index_a).unwrap();
         let index_b = u64::try_from(index_b).unwrap();
 
-        self.send_stream_ev(UpdatePlaylistEvents::PlaylistSwapTracks(PlaylistSwapInfo {
+        self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistSwapTracks(PlaylistSwapInfo {
             index_a,
             index_b,
         }));
@@ -542,8 +547,12 @@ impl Playlist {
         self.tracks.get(next_index)
     }
 
+    /// Set the [`RunningStatus`] of the playlist, also sends a stream event.
     pub fn set_status(&mut self, status: RunningStatus) {
         self.status = status;
+        self.send_stream_ev(UpdateEvents::PlayStateChanged {
+            playing: status.as_u32(),
+        });
     }
 
     #[must_use]
@@ -589,7 +598,7 @@ impl Playlist {
 
         self.loop_mode = new_mode;
 
-        self.send_stream_ev(UpdatePlaylistEvents::PlaylistLoopMode(
+        self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistLoopMode(
             PlaylistLoopModeInfo::from(self.loop_mode),
         ));
     }
@@ -647,7 +656,7 @@ impl Playlist {
 
         let url = track.as_podcast().unwrap().url();
 
-        self.send_stream_ev(UpdatePlaylistEvents::PlaylistAddTrack(
+        self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistAddTrack(
             PlaylistAddTrackInfo {
                 at_index: u64::try_from(self.tracks.len()).unwrap(),
                 title: track.title().map(ToOwned::to_owned),
@@ -701,7 +710,7 @@ impl Playlist {
 
         let track = Self::track_from_path(track_str)?;
 
-        self.send_stream_ev(UpdatePlaylistEvents::PlaylistAddTrack(
+        self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistAddTrack(
             PlaylistAddTrackInfo {
                 at_index: u64::try_from(self.tracks.len()).unwrap(),
                 title: track.title().map(ToOwned::to_owned),
@@ -716,30 +725,60 @@ impl Playlist {
         Ok(())
     }
 
+    /// Convert [`PlaylistTrackSource`] to [`Track`] by calling the correct functions.
+    ///
+    /// This mainly exist to de-duplicate this match and resulting error handling.
+    fn source_to_track(track_location: &PlaylistTrackSource, db_pod: &DBPod) -> Result<Track> {
+        let track = match track_location {
+            PlaylistTrackSource::Path(path) => Self::track_from_path(path)?,
+            PlaylistTrackSource::Url(uri) => Self::track_from_uri(uri),
+            PlaylistTrackSource::PodcastUrl(uri) => Self::track_from_podcasturi(uri, db_pod)?,
+        };
+
+        Ok(track)
+    }
+
     /// Add Paths / Urls from the music service
     ///
     /// # Errors
     ///
-    /// see [`Self::add_track`]
+    /// On error on a specific track, the error will be collected and the remaining tracks will be tried to be added.
+    ///
+    /// - if adding a track results in a error (path not found, unsupported file types, not enough permissions, etc)
     ///
     /// # Panics
     ///
     /// If `usize` cannot be converted to `u64`
-    pub fn add_tracks(&mut self, tracks: PlaylistAddTrack, db_pod: &DBPod) -> Result<()> {
+    pub fn add_tracks(
+        &mut self,
+        tracks: PlaylistAddTrack,
+        db_pod: &DBPod,
+    ) -> Result<(), PlaylistAddErrorCollection> {
         self.tracks.reserve(tracks.tracks.len());
         let at_index = usize::try_from(tracks.at_index).unwrap();
+        // collect non-fatal errors to continue adding the rest of the tracks
+        let mut errors: Vec<anyhow::Error> = Vec::new();
+
+        info!(
+            "Trying to add {} tracks to the playlist",
+            tracks.tracks.len()
+        );
+
+        let mut added_tracks = 0;
+
         if at_index >= self.len() {
             // insert tracks at the end
             for track_location in tracks.tracks {
-                let track = match &track_location {
-                    PlaylistTrackSource::Path(path) => Self::track_from_path(path)?,
-                    PlaylistTrackSource::Url(uri) => Self::track_from_uri(uri),
-                    PlaylistTrackSource::PodcastUrl(uri) => {
-                        Self::track_from_podcasturi(uri, db_pod)?
+                let track = match Self::source_to_track(&track_location, db_pod) {
+                    Ok(v) => v,
+                    Err(err) => {
+                        warn!("Error adding track: {err}");
+                        errors.push(err);
+                        continue;
                     }
                 };
 
-                self.send_stream_ev(UpdatePlaylistEvents::PlaylistAddTrack(
+                self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistAddTrack(
                     PlaylistAddTrackInfo {
                         at_index: u64::try_from(self.tracks.len()).unwrap(),
                         title: track.title().map(ToOwned::to_owned),
@@ -750,31 +789,41 @@ impl Playlist {
 
                 self.tracks.push(track);
                 self.is_modified = true;
+                added_tracks += 1;
             }
+        } else {
+            let mut at_index = at_index;
+            // insert tracks at position
+            for track_location in tracks.tracks {
+                let track = match Self::source_to_track(&track_location, db_pod) {
+                    Ok(v) => v,
+                    Err(err) => {
+                        warn!("Error adding track: {err}");
+                        errors.push(err);
+                        continue;
+                    }
+                };
 
-            return Ok(());
+                self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistAddTrack(
+                    PlaylistAddTrackInfo {
+                        at_index: u64::try_from(at_index).unwrap(),
+                        title: track.title().map(ToOwned::to_owned),
+                        duration: track.duration().unwrap_or_default(),
+                        trackid: track_location,
+                    },
+                ));
+
+                self.tracks.insert(at_index, track);
+                self.is_modified = true;
+                at_index += 1;
+                added_tracks += 1;
+            }
         }
-        let mut at_index = at_index;
-        // insert tracks at position
-        for track_location in tracks.tracks {
-            let track = match &track_location {
-                PlaylistTrackSource::Path(path) => Self::track_from_path(path)?,
-                PlaylistTrackSource::Url(uri) => Self::track_from_uri(uri),
-                PlaylistTrackSource::PodcastUrl(uri) => Self::track_from_podcasturi(uri, db_pod)?,
-            };
 
-            self.send_stream_ev(UpdatePlaylistEvents::PlaylistAddTrack(
-                PlaylistAddTrackInfo {
-                    at_index: u64::try_from(at_index).unwrap(),
-                    title: track.title().map(ToOwned::to_owned),
-                    duration: track.duration().unwrap_or_default(),
-                    trackid: track_location,
-                },
-            ));
+        info!("Added {} tracks with {} errors", added_tracks, errors.len());
 
-            self.tracks.insert(at_index, track);
-            self.is_modified = true;
-            at_index += 1;
+        if !errors.is_empty() {
+            return Err(PlaylistAddErrorCollection::from(errors));
         }
 
         Ok(())
@@ -795,11 +844,17 @@ impl Playlist {
         let at_index = usize::try_from(tracks.at_index).unwrap();
 
         if at_index >= self.tracks.len() {
-            bail!("at_index is higher than the length of the playlist! at_index is \"{at_index}\" and playlist length is \"{}\"", self.tracks.len());
+            bail!(
+                "at_index is higher than the length of the playlist! at_index is \"{at_index}\" and playlist length is \"{}\"",
+                self.tracks.len()
+            );
         }
 
         if at_index + tracks.tracks.len().saturating_sub(1) >= self.tracks.len() {
-            bail!("at_index + tracks to remove is higher than the length of the playlist! playlist lenght is \"{}\"", self.tracks.len());
+            bail!(
+                "at_index + tracks to remove is higher than the length of the playlist! playlist lenght is \"{}\"",
+                self.tracks.len()
+            );
         }
 
         for input_track in tracks.tracks {
@@ -814,7 +869,7 @@ impl Playlist {
             // verified that at index "at_index" the track is of the type and has the URI that was requested to be removed
             self.handle_remove(at_index);
 
-            self.send_stream_ev(UpdatePlaylistEvents::PlaylistRemoveTrack(
+            self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistRemoveTrack(
                 PlaylistRemoveTrackInfo {
                     at_index: u64::try_from(at_index).unwrap(),
                     trackid: input_track,
@@ -901,7 +956,7 @@ impl Playlist {
 
         self.handle_remove(index);
 
-        self.send_stream_ev(UpdatePlaylistEvents::PlaylistRemoveTrack(
+        self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistRemoveTrack(
             PlaylistRemoveTrackInfo {
                 at_index: u64::try_from(index).unwrap(),
                 trackid: track_source,
@@ -931,7 +986,7 @@ impl Playlist {
         self.current_track_index = 0;
         self.need_proceed_to_next = false;
 
-        self.send_stream_ev(UpdatePlaylistEvents::PlaylistCleared);
+        self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistCleared);
     }
 
     /// Shuffle the playlist
@@ -950,7 +1005,7 @@ impl Playlist {
             }
         }
 
-        self.send_stream_ev(UpdatePlaylistEvents::PlaylistShuffled(
+        self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistShuffled(
             PlaylistShuffledInfo {
                 tracks: self.as_grpc_playlist_tracks().unwrap(),
             },
@@ -1048,7 +1103,7 @@ impl Playlist {
                 let deleted_idx = self.tracks.len();
 
                 // NOTE: this function may send many events very quickly (for example on a folder delete), which could overwhelm the broadcast channel on a low capacity value
-                self.send_stream_ev(UpdatePlaylistEvents::PlaylistRemoveTrack(
+                self.send_stream_ev_pl(UpdatePlaylistEvents::PlaylistRemoveTrack(
                     PlaylistRemoveTrackInfo {
                         at_index: u64::try_from(deleted_idx).unwrap(),
                         trackid: track_source,
@@ -1112,8 +1167,8 @@ impl Playlist {
         self.next_track_index.is_some()
     }
 
-    /// Send stream events with consistent error handling
-    fn send_stream_ev(&self, ev: UpdatePlaylistEvents) {
+    /// Send Playlist stream events with consistent error handling
+    fn send_stream_ev_pl(&self, ev: UpdatePlaylistEvents) {
         // there is only one error case: no receivers
         if self
             .stream_tx
@@ -1123,6 +1178,14 @@ impl Playlist {
             debug!("Stream Event not send: No Receivers");
         }
     }
+
+    /// Send stream events with consistent error handling
+    fn send_stream_ev(&self, ev: UpdateEvents) {
+        // there is only one error case: no receivers
+        if self.stream_tx.send(ev).is_err() {
+            debug!("Stream Event not send: No Receivers");
+        }
+    }
 }
 
 const PLAYLIST_SAVE_FILENAME: &str = "playlist.log";
@@ -1134,6 +1197,38 @@ fn get_playlist_path() -> Result<PathBuf> {
     Ok(path)
 }
 
+/// Error collections for [`Playlist::add_tracks`].
+#[derive(Debug)]
+pub struct PlaylistAddErrorCollection {
+    pub errors: Vec<anyhow::Error>,
+}
+
+impl Display for PlaylistAddErrorCollection {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        writeln!(
+            f,
+            "There are {} Errors adding tracks to the playlist: [",
+            self.errors.len()
+        )?;
+
+        for err in &self.errors {
+            writeln!(f, "  {err},")?;
+        }
+
+        write!(f, "]")
+    }
+}
+
+impl Error for PlaylistAddErrorCollection {}
+
+impl From<Vec<anyhow::Error>> for PlaylistAddErrorCollection {
+    fn from(value: Vec<anyhow::Error>) -> Self {
+        Self {
+            errors: value.into_iter().map(|err| anyhow::anyhow!(err)).collect(),
+        }
+    }
+}
+
 // NOTE: this is not "thiserror" due to custom "Display" impl (the "Option" handling)
 /// Error for when [`Playlist::add_track`] fails
 #[derive(Debug)]
