@@ -1,9 +1,9 @@
 //! Module for all Logger related things
 
-use std::{backtrace::Backtrace, path::PathBuf};
+use std::backtrace::Backtrace;
 
 use colored::{Color, Colorize};
-use flexi_logger::{style, DeferredNow, FileSpec, Logger, LoggerHandle, Record};
+use flexi_logger::{DeferredNow, FileSpec, Logger, LoggerHandle, Record, style};
 
 use crate::cli::Args;
 
@@ -19,6 +19,7 @@ pub fn setup(args: &Args) -> LoggerHandle {
                 log_format,
                 color_log_format,
             ))
+            .panic_if_error_channel_is_broken(false)
             .log_to_stderr();
 
         if args.log_options.log_to_file {
@@ -28,23 +29,8 @@ pub fn setup(args: &Args) -> LoggerHandle {
                 logger = logger.format_for_files(log_format);
             }
 
-            // workaround for https://github.com/emabee/flexi_logger/issues/194
-            let path = if args
-                .log_options
-                .log_file
-                .parent()
-                .is_some_and(|v| !v.to_string_lossy().is_empty())
-            {
-                args.log_options.log_file.clone()
-            } else {
-                let mut path = PathBuf::from(".");
-                path.push(&args.log_options.log_file);
-
-                path
-            };
-
-            let filespec =
-                FileSpec::try_from(&path).expect("Expected logging file to be parsed correctly");
+            let filespec = FileSpec::try_from(&args.log_options.log_file)
+                .expect("Expected logging file to be parsed correctly");
             logger = logger
                 .log_to_file(filespec)
                 .append()
