@@ -8,13 +8,13 @@ use std::path::{Path, PathBuf};
 use std::sync::Arc;
 use std::time::Duration;
 
-use anyhow::{bail, Context as _, Result};
+use anyhow::{Context as _, Result, bail};
 use clap::Parser;
 use music_player_service::MusicPlayerService;
 use parking_lot::Mutex;
 use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
 use termusiclib::config::v2::server::{ComProtocol, ScanDepth};
-use termusiclib::config::{new_shared_server_settings, ServerOverlay, SharedServerSettings};
+use termusiclib::config::{ServerOverlay, SharedServerSettings, new_shared_server_settings};
 use termusiclib::player::music_player_server::MusicPlayerServer;
 use termusiclib::player::{GetProgressResponse, PlayerProgress, PlayerTime, RunningStatus};
 use termusiclib::track::MediaTypesSimple;
@@ -29,8 +29,8 @@ use tokio::sync::{broadcast, oneshot};
 use tokio::task::JoinHandle;
 use tokio::time::Instant;
 use tokio_util::sync::CancellationToken;
-use tonic::transport::server::TcpIncoming;
 use tonic::transport::Server;
+use tonic::transport::server::TcpIncoming;
 
 #[macro_use]
 extern crate log;
@@ -40,19 +40,16 @@ pub const VOLUME_STEP: VolumeSigned = 5;
 pub const SPEED_STEP: SpeedSigned = 1;
 
 /// The Limit of continues errors before stopping playback and awaiting user input to start something specific again.
-// SAFETY: using "unsafe" here as "const unwrap" is MSRV 1.83, we are currently on 1.82
-const BACKEND_ERROR_LIMIT: NonZeroUsize = unsafe { NonZeroUsize::new_unchecked(5) };
+const BACKEND_ERROR_LIMIT: NonZeroUsize = NonZeroUsize::new(5).unwrap();
 
 /// Stats for the music player responses
 #[derive(Debug, Clone, PartialEq)]
 struct PlayerStats {
     pub progress: PlayerProgress,
     pub current_track_index: u64,
-    pub status: u32,
     pub volume: u16,
     pub speed: i32,
     pub gapless: bool,
-    pub current_track_updated: bool,
     pub radio_title: String,
 }
 
@@ -64,24 +61,21 @@ impl PlayerStats {
                 total_duration: None,
             },
             current_track_index: 0,
-            status: 1,
             volume: 0,
             speed: 10,
             gapless: true,
-            current_track_updated: false,
             radio_title: String::new(),
         }
     }
 
-    pub fn as_getprogress_response(&self) -> GetProgressResponse {
+    pub fn as_getprogress_response(&self, status: RunningStatus) -> GetProgressResponse {
         GetProgressResponse {
             progress: Some(self.as_playertime()),
             current_track_index: self.current_track_index,
-            status: self.status,
+            status: status.as_u32(),
             volume: u32::from(self.volume),
             speed: self.speed,
             gapless: self.gapless,
-            current_track_updated: self.current_track_updated,
             radio_title: self.radio_title.clone(),
         }
     }
@@ -500,7 +494,6 @@ fn player_loop(
                 player.mpris_handle_events();
                 let mut p_tick = playerstats.lock();
                 let mut playlist = player.playlist.read();
-                p_tick.status = playlist.status().as_u32();
                 // branch to auto-start playing if status is "stopped"(not paused) and playlist is not empty anymore
                 if playlist.status() == RunningStatus::Stopped {
                     if playlist.is_empty() {
@@ -525,7 +518,7 @@ fn player_loop(
                     // the following function is "mut", which does not like having the immutable borrow to "playlist"
                     // so we have to unlock first then later re-acquire the handle for later parts
                     drop(playlist);
-                    player.mpris_update_progress(&p_tick.progress);
+                    player.update_progress(&p_tick.progress);
 
                     // only reset errors if position is either above 0 or total duration is available and is above 0
                     if pl_status == RunningStatus::Running
@@ -540,7 +533,6 @@ fn player_loop(
                 if player.current_track_updated {
                     p_tick.current_track_index =
                         u64::try_from(playlist.get_current_track_index()).unwrap();
-                    p_tick.current_track_updated = player.current_track_updated;
                     player.current_track_updated = false;
                 }
                 if let Some(track) = playlist.current_track() {
@@ -580,8 +572,6 @@ fn player_loop(
             PlayerCmd::TogglePause => {
                 info!("player toggled pause");
                 player.toggle_pause();
-                let mut p_tick = playerstats.lock();
-                p_tick.status = player.playlist.read().status().as_u32();
             }
             PlayerCmd::VolumeDown => {
                 info!("before volumedown: {}", player.volume());
@@ -719,7 +709,7 @@ fn get_config(args: &cli::Args) -> Result<ServerOverlay> {
         settings: config,
         music_dir_overwrite: music_dir,
         disable_discord_status: args.disable_discord,
-        library_scan_depth: max_depth,
+        metadata_scan_depth: max_depth,
     };
 
     Ok(overlay)
