@@ -1,5 +1,3 @@
-use termusiclib::config::SharedTuiSettings;
-use termusiclib::ids::Id;
 /**
  * MIT License
  *
@@ -23,15 +21,17 @@ use termusiclib::ids::Id;
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-use termusiclib::types::Msg;
+use termusiclib::config::SharedTuiSettings;
 use tui_realm_stdlib::Paragraph;
 use tuirealm::{
+    Component, Event, MockComponent,
     event::{Key, KeyEvent},
     props::{Alignment, BorderType, Borders, Color, TextModifiers, TextSpan},
-    Component, Event, MockComponent,
 };
 
+use crate::ui::ids::Id;
 use crate::ui::model::{Model, UserEvent};
+use crate::ui::msg::{ErrorPopupMsg, Msg};
 
 #[derive(MockComponent)]
 pub struct ErrorPopup {
@@ -57,7 +57,7 @@ impl ErrorPopup {
                 // .background(Color::Black)
                 .modifiers(TextModifiers::BOLD)
                 .alignment(Alignment::Center)
-                .text(&[TextSpan::from(msg)]/* &msg.lines().map(|v| TextSpan::from(v)).collect::<Vec<_>>() */),
+                .text([TextSpan::from(msg)]/* &msg.lines().map(|v| TextSpan::from(v)).collect::<Vec<_>>() */),
                 config
         }
     }
@@ -71,9 +71,13 @@ impl Component<Msg, UserEvent> for ErrorPopup {
             Event::Keyboard(KeyEvent {
                 code: Key::Enter | Key::Esc,
                 ..
-            }) => Some(Msg::ErrorPopupClose),
-            Event::Keyboard(key) if key == keys.quit.get() => Some(Msg::ErrorPopupClose),
-            Event::Keyboard(key) if key == keys.escape.get() => Some(Msg::ErrorPopupClose),
+            }) => Some(Msg::ErrorPopup(ErrorPopupMsg::Close)),
+            Event::Keyboard(key) if key == keys.quit.get() => {
+                Some(Msg::ErrorPopup(ErrorPopupMsg::Close))
+            }
+            Event::Keyboard(key) if key == keys.escape.get() => {
+                Some(Msg::ErrorPopup(ErrorPopupMsg::Close))
+            }
             _ => None,
         }
     }
@@ -83,14 +87,15 @@ impl Model {
     /// Mount error and give focus to it
     // This should likely be refactored to be "std::error::Error", but see https://github.com/dtolnay/anyhow/issues/63 on why it was easier this way
     pub fn mount_error_popup<E: Into<anyhow::Error>>(&mut self, err: E) {
-        assert!(self
-            .app
-            .remount(
-                Id::ErrorPopup,
-                Box::new(ErrorPopup::new(self.config_tui.clone(), err)),
-                vec![]
-            )
-            .is_ok());
+        assert!(
+            self.app
+                .remount(
+                    Id::ErrorPopup,
+                    Box::new(ErrorPopup::new(self.config_tui.clone(), err)),
+                    vec![]
+                )
+                .is_ok()
+        );
         assert!(self.app.active(&Id::ErrorPopup).is_ok());
     }
 
