@@ -1,21 +1,20 @@
-//! SPDX-License-Identifier: MIT
-
 use std::path::Path;
 
-use anyhow::{anyhow, bail, Result};
+use anyhow::{Result, anyhow, bail};
 use termusiclib::config::{SharedTuiSettings, TuiOverlay};
-use termusiclib::ids::Id;
 use termusiclib::track::MediaTypes;
-use termusiclib::types::{GSMsg, Msg};
-use tui_realm_stdlib::{Input, Table};
+use tui_realm_stdlib::Table;
 use tui_realm_treeview::TREE_INITIAL_NODE;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers};
 use tuirealm::props::{Alignment, BorderType, Borders, InputType, TableBuilder, TextSpan};
 use tuirealm::{AttrValue, Attribute, Component, Event, MockComponent, State, StateValue};
 
-use crate::ui::model::UserEvent;
 use crate::ui::Model;
+use crate::ui::components::vendored::tui_realm_stdlib_input::Input;
+use crate::ui::ids::Id;
+use crate::ui::model::UserEvent;
+use crate::ui::msg::{GSMsg, Msg};
 
 #[derive(MockComponent)]
 pub struct GSInputPopup {
@@ -92,7 +91,7 @@ impl Component<Msg, UserEvent> for GSInputPopup {
                 code: Key::Enter, ..
             }) => self.perform(Cmd::Submit),
             Event::Keyboard(KeyEvent { code: Key::Tab, .. }) => {
-                return Some(Msg::GeneralSearch(GSMsg::InputBlur))
+                return Some(Msg::GeneralSearch(GSMsg::InputBlur));
             }
             _ => CmdResult::None,
         };
@@ -176,7 +175,7 @@ impl GSTablePopup {
                 .rewind(false)
                 .step(4)
                 .row_height(1)
-                .headers(&["idx", "File name"])
+                .headers(["idx", "File name"])
                 .column_spacing(3)
                 .widths(&[5, 95])
                 .table(
@@ -201,7 +200,7 @@ impl GSTablePopup {
                 .rewind(false)
                 .step(4)
                 .row_height(1)
-                .headers(&["Duration", "Artist", "Title"])
+                .headers(["Duration", "Artist", "Title"])
                 .column_spacing(3)
                 .widths(&[14, 30, 56])
                 .table(
@@ -225,7 +224,7 @@ impl GSTablePopup {
                 .rewind(false)
                 .step(4)
                 .row_height(1)
-                .headers(&["Duration", "Artist", "Title"])
+                .headers(["Duration", "Artist", "Title"])
                 .column_spacing(3)
                 .widths(&[14, 30, 56])
                 .table(
@@ -249,7 +248,7 @@ impl GSTablePopup {
                 .rewind(false)
                 .step(4)
                 .row_height(1)
-                .headers(&["idx", "Episode Title"])
+                .headers(["idx", "Episode Title"])
                 .column_spacing(3)
                 .widths(&[5, 95])
                 .table(
@@ -273,7 +272,7 @@ impl GSTablePopup {
                 .rewind(false)
                 .step(4)
                 .row_height(1)
-                .headers(&["idx", "Podcast Title"])
+                .headers(["idx", "Podcast Title"])
                 .column_spacing(3)
                 .widths(&[5, 95])
                 .table(
@@ -299,10 +298,10 @@ impl Component<Msg, UserEvent> for GSTablePopup {
         let keys = &config.read().settings.keys;
         let cmd_result = match ev {
             Event::Keyboard(KeyEvent { code: Key::Esc, .. }) => {
-                return Some(Msg::GeneralSearch(GSMsg::PopupCloseCancel))
+                return Some(Msg::GeneralSearch(GSMsg::PopupCloseCancel));
             }
             Event::Keyboard(keyevent) if keyevent == keys.quit.get() => {
-                return Some(Msg::GeneralSearch(GSMsg::PopupCloseCancel))
+                return Some(Msg::GeneralSearch(GSMsg::PopupCloseCancel));
             }
 
             Event::Keyboard(KeyEvent { code: Key::Up, .. }) => {
@@ -332,26 +331,29 @@ impl Component<Msg, UserEvent> for GSTablePopup {
             Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.goto_bottom.get() => {
                 self.perform(Cmd::GoTo(Position::End))
             }
+            Event::Keyboard(KeyEvent {
+                code: Key::Home, ..
+            }) => self.perform(Cmd::GoTo(Position::Begin)),
             Event::Keyboard(KeyEvent { code: Key::End, .. }) => {
                 self.perform(Cmd::GoTo(Position::End))
             }
             Event::Keyboard(KeyEvent { code: Key::Tab, .. }) => {
-                return Some(Msg::GeneralSearch(GSMsg::TableBlur))
+                return Some(Msg::GeneralSearch(GSMsg::TableBlur));
             }
 
             Event::Keyboard(keyevent) if keyevent == keys.navigation_keys.right.get() => {
                 match self.source {
                     Source::Library => {
-                        return Some(Msg::GeneralSearch(GSMsg::PopupCloseLibraryAddPlaylist))
+                        return Some(Msg::GeneralSearch(GSMsg::PopupCloseLibraryAddPlaylist));
                     }
                     Source::Playlist => {
-                        return Some(Msg::GeneralSearch(GSMsg::PopupClosePlaylistPlaySelected))
+                        return Some(Msg::GeneralSearch(GSMsg::PopupClosePlaylistPlaySelected));
                     }
                     Source::Database => {
-                        return Some(Msg::GeneralSearch(GSMsg::PopupCloseDatabaseAddPlaylist))
+                        return Some(Msg::GeneralSearch(GSMsg::PopupCloseDatabaseAddPlaylist));
                     }
                     Source::Episode => {
-                        return Some(Msg::GeneralSearch(GSMsg::PopupCloseEpisodeAddPlaylist))
+                        return Some(Msg::GeneralSearch(GSMsg::PopupCloseEpisodeAddPlaylist));
                     }
                     Source::Podcast => return None,
                 }
@@ -360,17 +362,17 @@ impl Component<Msg, UserEvent> for GSTablePopup {
                 code: Key::Enter, ..
             }) => match self.source {
                 Source::Library => {
-                    return Some(Msg::GeneralSearch(GSMsg::PopupCloseOkLibraryLocate))
+                    return Some(Msg::GeneralSearch(GSMsg::PopupCloseOkLibraryLocate));
                 }
                 Source::Playlist => {
-                    return Some(Msg::GeneralSearch(GSMsg::PopupCloseOkPlaylistLocate))
+                    return Some(Msg::GeneralSearch(GSMsg::PopupCloseOkPlaylistLocate));
                 }
                 Source::Database => return Some(Msg::GeneralSearch(GSMsg::PopupCloseCancel)),
                 Source::Episode => {
-                    return Some(Msg::GeneralSearch(GSMsg::PopupCloseOkEpisodeLocate))
+                    return Some(Msg::GeneralSearch(GSMsg::PopupCloseOkEpisodeLocate));
                 }
                 Source::Podcast => {
-                    return Some(Msg::GeneralSearch(GSMsg::PopupCloseOkPodcastLocate))
+                    return Some(Msg::GeneralSearch(GSMsg::PopupCloseOkPodcastLocate));
                 }
             },
             _ => CmdResult::None,
@@ -400,14 +402,15 @@ impl Model {
                 if let Some(line) = table.get(index) {
                     if let Some(text_span) = line.get(1) {
                         let node = text_span.content.clone();
-                        assert!(self
-                            .app
-                            .attr(
-                                &Id::Library,
-                                Attribute::Custom(TREE_INITIAL_NODE),
-                                AttrValue::String(node),
-                            )
-                            .is_ok());
+                        assert!(
+                            self.app
+                                .attr(
+                                    &Id::Library,
+                                    Attribute::Custom(TREE_INITIAL_NODE),
+                                    AttrValue::String(node),
+                                )
+                                .is_ok()
+                        );
                     }
                 }
             }
