@@ -1,37 +1,17 @@
-/**
- * MIT License
- *
- * termusic - Copyright (C) 2021 Larry Hao
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in all
- * copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
- */
+use anyhow::anyhow;
+use termusiclib::songtag::TrackDLMsg;
+
 use crate::ui::Model;
-use termusiclib::ids::{Id, IdTagEditor};
-use termusiclib::types::{TEMsg, TFMsg};
+use crate::ui::ids::{Id, IdTagEditor};
+use crate::ui::msg::{TEMsg, TFMsg};
 
 impl Model {
     pub fn update_tageditor(&mut self, msg: TEMsg) {
         match msg {
-            TEMsg::TagEditorRun(node_id) => {
+            TEMsg::Open(node_id) => {
                 self.mount_tageditor(&node_id);
             }
-            TEMsg::TagEditorClose => {
+            TEMsg::Close => {
                 if let Some(s) = self.tageditor_song.clone() {
                     // TODO: this should be re-done and take actual track ids themself, or at least verified to use the same functions to result in the same id
                     self.library_reload_with_node_focus(Some(s.path_as_id_str().to_string()));
@@ -39,37 +19,47 @@ impl Model {
                 self.umount_tageditor();
             }
 
-            TEMsg::TECounterDeleteOk => {
+            TEMsg::CounterDeleteOk => {
                 self.te_delete_lyric();
             }
-            TEMsg::TESelectLyricOk(index) => {
+            TEMsg::SelectLyricOk(index) => {
                 if let Some(mut song) = self.tageditor_song.take() {
                     song.set_lyric_selected_index(index);
                     // the unwrap should also never happen as all components should be properly mounted
                     self.init_by_song(song).unwrap();
                 }
             }
-            TEMsg::TESearch => {
+            TEMsg::Search => {
                 self.te_songtag_search();
             }
-            TEMsg::TEDownload(index) => {
+            TEMsg::Download(index) => {
                 if let Err(e) = self.te_songtag_download(index) {
                     self.mount_error_popup(e.context("download by songtag"));
                 }
             }
-            TEMsg::TEEmbed(index) => {
+            TEMsg::Embed(index) => {
                 if let Err(e) = self.te_load_lyric_and_photo(index) {
                     self.mount_error_popup(e.context("log lyric and photo"));
                 }
             }
-            TEMsg::TERename => {
+            TEMsg::EmbedDone(song) => {
+                self.te_load_lyric_and_photo_done(song);
+            }
+            TEMsg::EmbedErr(err) => {
+                self.mount_error_popup(anyhow!(err));
+            }
+            TEMsg::Save => {
                 if let Err(e) = self.te_rename_song_by_tag() {
                     self.mount_error_popup(e.context("rename song by tag"));
                 }
             }
-            TEMsg::TEFocus(m) => self.update_tag_editor_focus(m),
+            TEMsg::Focus(msg) => self.update_tag_editor_focus(msg),
 
-            TEMsg::TESearchLyricResult(m) => self.te_update_lyric_results(m),
+            TEMsg::SearchLyricResult(msg) => self.te_update_lyric_results(msg),
+            TEMsg::TrackDownloadResult(msg) => self.te_update_download_msg(msg),
+            TEMsg::TrackDownloadPreError(err) => {
+                self.mount_error_popup(anyhow!(err));
+            }
         }
     }
 
@@ -117,4 +107,59 @@ impl Model {
             }
         }
     }
+
+    /// Handle all cases for [`TrackDLMsg`].
+    fn te_update_download_msg(&mut self, msg: TrackDLMsg) {
+        match msg {
+            TrackDLMsg::Start(url, title) => {
+                self.download_tracker.increase_one(&*url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_download_start(&title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            TrackDLMsg::Success(url) => {
+                self.download_tracker.decrease_one(&url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_download_complete(),
+                    None,
+                    None,
+                    None,
+                );
+
+                if self.app.mounted(&Id::TagEditor(IdTagEditor::LabelHint)) {
+                    self.umount_tageditor();
+                }
+            }
+            TrackDLMsg::Completed(_url, file) => {
+                if self.download_tracker.visible() {
+                    return;
+                }
+                self.library_reload_with_node_focus(file);
+            }
+            TrackDLMsg::Err(url, title, error_message) => {
+                self.download_tracker.decrease_one(&url);
+                self.mount_error_popup(anyhow!("download failed: {error_message}"));
+                self.show_message_timeout_label_help(
+                    self.download_tracker
+                        .message_download_error_response(&title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            TrackDLMsg::ErrEmbedData(_url, title) => {
+                self.mount_error_popup(anyhow!("download ok but tag info is not complete."));
+                self.show_message_timeout_label_help(
+                    self.download_tracker
+                        .message_download_error_embed_data(&title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+        }
+    }
 }
