@@ -1,47 +1,49 @@
-//! SPDX-License-Identifier: MIT
-
 use std::path::PathBuf;
 use std::str::FromStr;
 use std::sync::atomic::Ordering;
-use std::time::{Duration, Instant};
+use std::time::Duration;
 
-use anyhow::{anyhow, bail, Context, Result};
+use anyhow::{Context, Result, anyhow, bail};
 use id3::frame::Lyrics as Id3Lyrics;
+use termusiclib::config::v2::server::ScanDepth;
 use termusiclib::config::v2::tui::keys::Keys;
 use termusiclib::config::v2::tui::theme::ThemeWrap;
 use termusiclib::config::{ServerOverlay, SharedServerSettings, SharedTuiSettings};
-use termusiclib::ids::Id;
-use termusiclib::library_db::TrackDB;
-use termusiclib::library_db::{DataBase, SearchCriteria};
+use termusiclib::new_database::Database;
+use termusiclib::new_database::track_ops::TrackRead;
 use termusiclib::player::playlist_helpers::PlaylistTrackSource;
 use termusiclib::player::{PlaylistTracks, RunningStatus};
-use termusiclib::podcast::{db::Database as DBPod, Podcast, PodcastFeed};
-use termusiclib::songtag::lrc::Lyric;
+use termusiclib::podcast::{Podcast, PodcastFeed, db::Database as DBPod};
 use termusiclib::songtag::SongTag;
+use termusiclib::songtag::lrc::Lyric;
 use termusiclib::taskpool::TaskPool;
 use termusiclib::track::{LyricData, MediaTypesSimple, Track};
-use termusiclib::types::{Msg, YoutubeOptions};
 #[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
 use termusiclib::ueberzug::UeInstance;
 use termusiclib::utils::get_app_config_path;
 use termusiclib::xywh;
-use tokio::sync::mpsc::{unbounded_channel, UnboundedReceiver, UnboundedSender};
+use tokio::sync::mpsc::{UnboundedSender, unbounded_channel};
 use tui_realm_treeview::Tree;
 use tuirealm::terminal::{CrosstermTerminalAdapter, TerminalBridge};
 
 use super::components::TETrack;
 use super::tui_cmd::TuiCmd;
-use crate::ui::Application;
 use crate::CombinedSettings;
+use crate::ui::Application;
+use crate::ui::ids::Id;
+use crate::ui::model::ports::stream_events::{PortStreamEvents, WrappedStreamEvents};
+use crate::ui::model::youtube_options::YoutubeOptions;
+use crate::ui::msg::{Msg, SearchCriteria};
 pub use download_tracker::DownloadTracker;
 pub use user_events::UserEvent;
 
 mod download_tracker;
 mod playlist;
+mod ports;
 mod update;
 mod user_events;
 mod view;
-mod youtube_options;
+pub mod youtube_options;
 
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum TermusicLayout {
@@ -77,7 +79,7 @@ pub struct DatabaseWidgetData {
     /// Criteria Search results `(criteria -> this)`
     pub search_results: Vec<String>,
     /// Results of the critea results search `(criteria -> search_results -> this)`
-    pub search_tracks: Vec<TrackDB>,
+    pub search_tracks: Vec<TrackRead>,
 }
 
 impl DatabaseWidgetData {
@@ -289,18 +291,16 @@ pub struct Model {
     pub quit: bool,
     /// Tells whether to redraw interface
     pub redraw: bool,
-    last_redraw: Instant,
     pub app: Application<Id, Msg, UserEvent>,
     /// Used to draw to terminal
     pub terminal: TerminalBridge<CrosstermTerminalAdapter>,
     pub tx_to_main: TxToMain,
-    pub rx_to_main: UnboundedReceiver<Msg>,
     /// Sender for Player Commands
     pub cmd_to_server_tx: UnboundedSender<TuiCmd>,
 
     pub config_tui: SharedTuiSettings,
     pub config_server: SharedServerSettings,
-    pub db: DataBase,
+    pub db: Database,
 
     pub layout: TermusicLayout,
     pub library: MusicLibraryData,
@@ -357,7 +357,11 @@ fn get_viuer_support() -> ViuerSupported {
 
 impl Model {
     #[allow(clippy::too_many_lines)]
-    pub async fn new(config: CombinedSettings, cmd_to_server_tx: UnboundedSender<TuiCmd>) -> Self {
+    pub fn new(
+        config: CombinedSettings,
+        cmd_to_server_tx: UnboundedSender<TuiCmd>,
+        stream_updates: WrappedStreamEvents,
+    ) -> Self {
         let CombinedSettings {
             server: config_server,
             tui: config_tui,
@@ -374,7 +378,7 @@ impl Model {
 
         info!("Using viuer protocol {viuer_supported:#?}");
 
-        let db = DataBase::new(&config_server.read()).expect("Open Library Database");
+        let db = Database::new_default_path().expect("Open Library Database");
         let db_criteria = SearchCriteria::Artist;
         let terminal = TerminalBridge::new_crossterm().expect("Could not initialize terminal");
 
@@ -403,7 +407,9 @@ impl Model {
         ));
         let (tx_to_main, rx_to_main) = unbounded_channel();
 
-        let app = Self::init_app(&tree, &config_tui);
+        let stream_update_port = PortStreamEvents::new(stream_updates);
+
+        let app = Self::init_app(&tree, &config_tui, rx_to_main, stream_update_port);
 
         // This line is required, in order to show the playing message for the first track
         // playlist.set_current_track_index(0);
@@ -417,7 +423,7 @@ impl Model {
             tx_to_main.clone(),
             download_tracker.clone(),
             &path,
-            config_server.read().get_library_scan_depth(),
+            ScanDepth::Limited(2),
             None,
         );
 
@@ -425,7 +431,6 @@ impl Model {
             app,
             quit: false,
             redraw: true,
-            last_redraw: Instant::now(),
             terminal,
             config_server,
             config_tui,
@@ -437,11 +442,7 @@ impl Model {
                 tree,
                 yanked_node_id: None,
             },
-            // TODO: Consider making YoutubeOptions async and use async reqwest in YoutubeOptions
-            // and avoid this `spawn_blocking` call.
-            youtube_options: tokio::task::spawn_blocking(YoutubeOptions::default)
-                .await
-                .expect("Failed to initialize YoutubeOptions in a blocking task due to a panic"),
+            youtube_options: YoutubeOptions::default(),
             #[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
             ueberzug_instance,
             songtag_options: vec![],
@@ -468,7 +469,6 @@ impl Model {
             },
             taskpool,
             tx_to_main,
-            rx_to_main,
             download_tracker,
             current_track_lyric: None,
             playback: Playback::new(),
@@ -503,7 +503,15 @@ impl Model {
             self.mount_error_popup(e.context("theme save"));
         }
         self.mount_label_help();
-        self.db.sync_database(&self.library.tree_path);
+        if let Err(err) =
+            self.db
+                .scan_path(&self.library.tree_path, &self.config_server.read(), false)
+        {
+            error!(
+                "Error scanning path {:#?}: {err:#?}",
+                self.library.tree_path.display()
+            );
+        }
         self.playlist_sync();
     }
 
@@ -538,30 +546,25 @@ impl Model {
         crate::TERMINAL_ALTERNATE_MODE.store(false, Ordering::SeqCst);
     }
 
-    /// Returns elapsed time since last redraw
-    pub fn since_last_redraw(&self) -> Duration {
-        self.last_redraw.elapsed()
-    }
-
     /// Force a redraw of the entire model
     pub fn force_redraw(&mut self) {
         self.redraw = true;
     }
 
     /// Send a command to request the Track Progress and set the titles to the current state.
-    pub fn run(&mut self) {
+    #[inline]
+    pub fn request_progress(&mut self) {
         self.command(TuiCmd::GetProgress);
-        self.progress_update_title();
-        self.lyric_update_title();
     }
 
+    /// Update all the places that need to be updated after a current track change or running status change.
     pub fn player_update_current_track_after(&mut self) {
-        self.playback.set_current_track_pos(Duration::ZERO);
         if let Err(e) = self.update_photo() {
             self.mount_error_popup(e.context("update_photo"));
         }
         self.progress_update_title();
         self.lyric_update_title();
+        self.lyric_update();
         self.update_playing_song();
     }
 
