@@ -1,22 +1,24 @@
-//! SPDX-License-Identifier: MIT
-
-use std::path::{Path, PathBuf};
+use std::path::Path;
 use std::time::Duration;
 
-use anyhow::anyhow;
-use termusiclib::ids::{Id, IdTagEditor};
+use anyhow::{Result, anyhow};
+use termusiclib::player::{PlayerProgress, RunningStatus, UpdateEvents, UpdatePlaylistEvents};
+use termusiclib::podcast::{PodcastDLResult, PodcastSyncResult};
 use termusiclib::track::MediaTypesSimple;
-use termusiclib::types::{
-    DBMsg, DLMsg, GSMsg, LIMsg, LyricMsg, MainLayoutMsg, Msg, PCMsg, PLMsg, PlayerMsg,
-    SavePlaylistMsg, XYWHMsg, YSMsg,
-};
 use tokio::runtime::Handle;
 use tokio::time::sleep;
-use tuirealm::props::{AttrValue, Attribute};
 use tuirealm::Update;
+use tuirealm::props::{AttrValue, Attribute};
 
+use crate::ui::ids::Id;
+use crate::ui::model::youtube_options::YTDLMsg;
+use crate::ui::msg::{
+    CoverDLResult, DBMsg, DeleteConfirmMsg, ErrorPopupMsg, GSMsg, HelpPopupMsg, LIMsg, LyricMsg,
+    MainLayoutMsg, Msg, NotificationMsg, PCMsg, PLMsg, PlayerMsg, QuitPopupMsg, SavePlaylistMsg,
+    ServerReqResponse, XYWHMsg, YSMsg,
+};
 use crate::ui::tui_cmd::TuiCmd;
-use crate::ui::{model::TermusicLayout, Model};
+use crate::ui::{Model, model::TermusicLayout};
 
 impl Update<Msg> for Model {
     #[allow(clippy::too_many_lines)]
@@ -26,75 +28,36 @@ impl Update<Msg> for Model {
         self.redraw = true;
         // Match message
         match msg {
-            Msg::ConfigEditor(m) => self.update_config_editor(m),
-            Msg::DataBase(m) => self.update_database_list(m),
+            Msg::ConfigEditor(msg) => self.update_config_editor(msg),
+            Msg::DataBase(msg) => self.update_database_list(msg),
 
-            Msg::DeleteConfirmShow | Msg::DeleteConfirmCloseCancel | Msg::DeleteConfirmCloseOk => {
-                self.update_delete_confirmation(&msg)
-            }
+            Msg::DeleteConfirm(msg) => self.update_delete_confirmation(&msg),
 
-            Msg::ErrorPopupClose => {
-                if self.app.mounted(&Id::ErrorPopup) {
-                    self.umount_error_popup();
-                }
+            Msg::ErrorPopup(msg) => self.update_error_popup_msg(&msg),
+            Msg::QuitPopup(msg) => self.update_quit_popup_msg(&msg),
+
+            Msg::Library(msg) => {
+                self.update_library(msg);
                 None
             }
-            Msg::QuitPopupShow => {
-                if self.config_tui.read().settings.behavior.confirm_quit {
-                    self.mount_quit_popup();
-                } else {
-                    self.quit = true;
-                }
+            Msg::GeneralSearch(msg) => {
+                self.update_general_search(&msg);
                 None
             }
-            Msg::QuitPopupCloseCancel => {
-                self.app.umount(&Id::QuitPopup).ok();
-                None
-            }
-            Msg::QuitPopupCloseOk => {
-                self.quit = true;
-                None
-            }
-            Msg::Library(m) => {
-                self.update_library(m);
-                None
-            }
-            Msg::GeneralSearch(m) => {
-                self.update_general_search(&m);
-                None
-            }
-            Msg::Playlist(m) => {
-                self.update_playlist(&m);
+            Msg::Playlist(msg) => {
+                self.update_playlist(&msg);
                 None
             }
 
             Msg::Player(msg) => self.update_player(msg),
 
-            Msg::HelpPopupShow => {
-                self.mount_help_popup();
+            Msg::HelpPopup(msg) => self.update_help_popup_msg(&msg),
+            Msg::YoutubeSearch(msg) => {
+                self.update_youtube_search(msg);
                 None
             }
-            Msg::HelpPopupClose => {
-                if self.app.mounted(&Id::HelpPopup) {
-                    self.app.umount(&Id::HelpPopup).ok();
-                }
-                self.update_photo().ok();
-                None
-            }
-            Msg::YoutubeSearch(m) => {
-                self.update_youtube_search(m);
-                None
-            }
-            Msg::LyricCycle => {
-                self.lyric_cycle();
-                None
-            }
-            Msg::LyricAdjustDelay(offset) => {
-                self.lyric_adjust_delay(offset);
-                None
-            }
-            Msg::TagEditor(m) => {
-                self.update_tageditor(m);
+            Msg::TagEditor(msg) => {
+                self.update_tageditor(msg);
                 None
             }
             Msg::UpdatePhoto => {
@@ -107,10 +70,12 @@ impl Update<Msg> for Model {
 
             Msg::SavePlaylist(msg) => self.update_save_playlist(msg),
 
-            Msg::Podcast(m) => self.update_podcast(m),
-            Msg::LyricMessage(m) => self.update_lyric_textarea(m),
-            Msg::Download(m) => self.update_download_msg(m),
-            Msg::Xywh(m) => self.update_xywh_msg(m),
+            Msg::Podcast(msg) => self.update_podcast(msg),
+            Msg::LyricMessage(msg) => self.update_lyric_msg(msg),
+            Msg::Notification(msg) => self.update_notification_msg(msg),
+            Msg::Xywh(msg) => self.update_xywh_msg(msg),
+            Msg::ServerReqResponse(msg) => self.update_server_resp_msg(msg),
+            Msg::StreamUpdate(msg) => self.update_update_events_msg(msg),
 
             Msg::ForceRedraw => None,
         }
@@ -127,6 +92,58 @@ impl Model {
         }
     }
 
+    /// Handle all [`ErrorPopupMsg`] messages. Sub-function for [`update`](Self::update).
+    fn update_error_popup_msg(&mut self, msg: &ErrorPopupMsg) -> Option<Msg> {
+        match msg {
+            ErrorPopupMsg::Close => {
+                if self.app.mounted(&Id::ErrorPopup) {
+                    self.umount_error_popup();
+                }
+            }
+        }
+
+        None
+    }
+
+    /// Handle all [`HelpPopupMsg`] messages. Sub-function for [`update`](Self::update).
+    fn update_help_popup_msg(&mut self, msg: &HelpPopupMsg) -> Option<Msg> {
+        match msg {
+            HelpPopupMsg::Show => {
+                self.mount_help_popup();
+            }
+            HelpPopupMsg::Close => {
+                if self.app.mounted(&Id::HelpPopup) {
+                    self.app.umount(&Id::HelpPopup).ok();
+                }
+                self.update_photo().ok();
+            }
+        }
+
+        None
+    }
+
+    /// Handle all [`QuitPopupMsg`] messages. Sub-function for [`update`](Self::update).
+    fn update_quit_popup_msg(&mut self, msg: &QuitPopupMsg) -> Option<Msg> {
+        match msg {
+            QuitPopupMsg::Show => {
+                if self.config_tui.read().settings.behavior.confirm_quit {
+                    self.mount_quit_popup();
+                } else {
+                    self.quit = true;
+                }
+            }
+            QuitPopupMsg::CloseCancel => {
+                self.app.umount(&Id::QuitPopup).ok();
+            }
+            QuitPopupMsg::CloseOk => {
+                self.quit = true;
+            }
+        }
+
+        None
+    }
+
+    /// Handle all [`XYWHMsg`] messages. Sub-function for [`update`](Self::update).
     fn update_xywh_msg(&mut self, msg: XYWHMsg) -> Option<Msg> {
         match msg {
             XYWHMsg::MoveLeft => self.xywh_move_left(),
@@ -138,14 +155,31 @@ impl Model {
             XYWHMsg::ToggleHidden => {
                 self.xywh_toggle_hide();
             }
+            XYWHMsg::CoverDLResult(msg) => match msg {
+                CoverDLResult::FetchPhotoSuccess(image_wrapper) => {
+                    self.show_image(&image_wrapper.data).ok();
+                }
+                CoverDLResult::FetchPhotoErr(err_text) => {
+                    self.show_message_timeout_label_help(err_text, None, None, None);
+                }
+            },
         }
         None
     }
 
-    fn update_lyric_textarea(&mut self, msg: LyricMsg) -> Option<Msg> {
+    /// Handle all [`LyricMsg`] messages. Sub-function for [`update`](Self::update).
+    fn update_lyric_msg(&mut self, msg: LyricMsg) -> Option<Msg> {
         match msg {
-            LyricMsg::LyricTextAreaBlurUp => self.app.active(&Id::Playlist).ok(),
-            LyricMsg::LyricTextAreaBlurDown => match self.layout {
+            LyricMsg::Cycle => {
+                self.lyric_cycle();
+                None
+            }
+            LyricMsg::AdjustDelay(offset) => {
+                self.lyric_adjust_delay(offset);
+                None
+            }
+            LyricMsg::TextAreaBlurUp => self.app.active(&Id::Playlist).ok(),
+            LyricMsg::TextAreaBlurDown => match self.layout {
                 TermusicLayout::TreeView => self.app.active(&Id::Library).ok(),
                 TermusicLayout::DataBase => self.app.active(&Id::DBListCriteria).ok(),
                 TermusicLayout::Podcast => self.app.active(&Id::Podcast).ok(),
@@ -154,6 +188,21 @@ impl Model {
         None
     }
 
+    /// Handle all [`NotificationMsg`] messages. Sub-function for [`update`](Self::update).
+    fn update_notification_msg(&mut self, msg: NotificationMsg) -> Option<Msg> {
+        match msg {
+            NotificationMsg::MessageShow((title, text)) => {
+                self.mount_message(&title, &text);
+            }
+            NotificationMsg::MessageHide((title, text)) => {
+                self.umount_message(&title, &text);
+            }
+        }
+
+        None
+    }
+
+    /// Handle all [`PCMsg`] messages. Sub-function for [`update`](Self::update).
     #[allow(clippy::too_many_lines)]
     fn update_podcast(&mut self, msg: PCMsg) -> Option<Msg> {
         match msg {
@@ -181,40 +230,10 @@ impl Model {
                 }
             }
             PCMsg::PodcastAddPopupCloseCancel => self.umount_podcast_add_popup(),
-            PCMsg::SyncData((id, pod)) => {
-                self.download_tracker.decrease_one(&pod.url);
-                self.show_message_timeout_label_help(
-                    self.download_tracker.message_sync_success(),
-                    None,
-                    None,
-                    None,
-                );
-                if let Err(e) = self.add_or_sync_data(&pod, Some(id)) {
-                    self.mount_error_popup(e.context("add or sync data"));
-                }
-            }
-            PCMsg::NewData(pod) => {
-                self.download_tracker.decrease_one(&pod.url);
-                self.show_message_timeout_label_help(
-                    self.download_tracker.message_feeds_added(),
-                    None,
-                    None,
-                    None,
-                );
-                if let Err(e) = self.add_or_sync_data(&pod, None) {
-                    self.mount_error_popup(e.context("add or sync data"));
-                }
-            }
-            PCMsg::Error(feed) => {
-                self.download_tracker.decrease_one(&feed.url);
-                self.mount_error_popup(anyhow!("Error happened with feed: {:?}", feed.title));
-                self.show_message_timeout_label_help(
-                    self.download_tracker.message_feed_sync_failed(),
-                    None,
-                    None,
-                    None,
-                );
-            }
+
+            PCMsg::SyncResult(msg) => self.podcast_handle_sync_result(msg),
+            PCMsg::DLResult(msg) => self.podcast_handle_dl_result(msg),
+
             PCMsg::PodcastSelected(index) => {
                 self.podcast.podcasts_index = index;
                 if let Err(e) = self.podcast_sync_episodes() {
@@ -247,74 +266,13 @@ impl Model {
                     self.mount_error_popup(e.context("podcast refresh feeds all"));
                 }
             }
-            PCMsg::FetchPodcastStart(url) => {
-                self.download_tracker.increase_one(url);
-                self.show_message_timeout_label_help(
-                    self.download_tracker.message_sync_start(),
-                    None,
-                    None,
-                    None,
-                );
-            }
+
             PCMsg::EpisodeDownload(index) => {
                 if let Err(e) = self.episode_download(Some(index)) {
                     self.mount_error_popup(e.context("podcast episode download"));
                 }
             }
-            PCMsg::DLStart(ep_data) => {
-                self.download_tracker.increase_one(&ep_data.url);
-                self.show_message_timeout_label_help(
-                    self.download_tracker.message_download_start(&ep_data.title),
-                    None,
-                    None,
-                    None,
-                );
-            }
-            PCMsg::DLComplete(ep_data) => {
-                if let Err(e) = self.episode_download_complete(ep_data.clone()) {
-                    self.mount_error_popup(e.context("podcast episode download complete"));
-                }
-                self.download_tracker.decrease_one(&ep_data.url);
-                self.show_message_timeout_label_help(
-                    self.download_tracker.message_download_complete(),
-                    None,
-                    None,
-                    None,
-                );
-            }
-            PCMsg::DLResponseError(ep_data) => {
-                self.download_tracker.decrease_one(&ep_data.url);
-                self.mount_error_popup(anyhow!("download failed for episode: {}", ep_data.title));
-                self.show_message_timeout_label_help(
-                    self.download_tracker
-                        .message_download_error_response(&ep_data.title),
-                    None,
-                    None,
-                    None,
-                );
-            }
-            PCMsg::DLFileCreateError(ep_data) => {
-                self.download_tracker.decrease_one(&ep_data.url);
-                self.mount_error_popup(anyhow!("download failed for episode: {}", ep_data.title));
-                self.show_message_timeout_label_help(
-                    self.download_tracker
-                        .message_download_error_file_create(&ep_data.title),
-                    None,
-                    None,
-                    None,
-                );
-            }
-            PCMsg::DLFileWriteError(ep_data) => {
-                self.download_tracker.decrease_one(&ep_data.url);
-                self.mount_error_popup(anyhow!("download failed for episode: {}", ep_data.title));
-                self.show_message_timeout_label_help(
-                    self.download_tracker
-                        .message_download_error_file_write(&ep_data.title),
-                    None,
-                    None,
-                    None,
-                );
-            }
+
             PCMsg::EpisodeDeleteFile(index) => {
                 if let Err(e) = self.episode_delete_file(index) {
                     self.mount_error_popup(e.context("podcast episode delete"));
@@ -353,6 +311,115 @@ impl Model {
         None
     }
 
+    /// Handle all cases for [`PodcastSyncResult`].
+    fn podcast_handle_sync_result(&mut self, msg: PodcastSyncResult) {
+        match msg {
+            PodcastSyncResult::FetchPodcastStart(url) => {
+                self.download_tracker.increase_one(url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_sync_start(),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            PodcastSyncResult::SyncData((id, pod)) => {
+                self.download_tracker.decrease_one(&pod.url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_sync_success(),
+                    None,
+                    None,
+                    None,
+                );
+                if let Err(e) = self.add_or_sync_data(&pod, Some(id)) {
+                    self.mount_error_popup(e.context("add or sync data"));
+                }
+            }
+            PodcastSyncResult::NewData(pod) => {
+                self.download_tracker.decrease_one(&pod.url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_feeds_added(),
+                    None,
+                    None,
+                    None,
+                );
+                if let Err(e) = self.add_or_sync_data(&pod, None) {
+                    self.mount_error_popup(e.context("add or sync data"));
+                }
+            }
+            PodcastSyncResult::Error(feed) => {
+                self.download_tracker.decrease_one(&feed.url);
+                self.mount_error_popup(anyhow!("Error happened with feed: {:?}", feed.title));
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_feed_sync_failed(),
+                    None,
+                    None,
+                    None,
+                );
+            }
+        }
+    }
+
+    /// Handle all cases for [`PodcastDLResult`].
+    fn podcast_handle_dl_result(&mut self, msg: PodcastDLResult) {
+        match msg {
+            PodcastDLResult::DLStart(ep_data) => {
+                self.download_tracker.increase_one(&ep_data.url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_download_start(&ep_data.title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            PodcastDLResult::DLComplete(ep_data) => {
+                if let Err(e) = self.episode_download_complete(ep_data.clone()) {
+                    self.mount_error_popup(e.context("podcast episode download complete"));
+                }
+                self.download_tracker.decrease_one(&ep_data.url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_download_complete(),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            PodcastDLResult::DLResponseError(ep_data) => {
+                self.download_tracker.decrease_one(&ep_data.url);
+                self.mount_error_popup(anyhow!("download failed for episode: {}", ep_data.title));
+                self.show_message_timeout_label_help(
+                    self.download_tracker
+                        .message_download_error_response(&ep_data.title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            PodcastDLResult::DLFileCreateError(ep_data) => {
+                self.download_tracker.decrease_one(&ep_data.url);
+                self.mount_error_popup(anyhow!("download failed for episode: {}", ep_data.title));
+                self.show_message_timeout_label_help(
+                    self.download_tracker
+                        .message_download_error_file_create(&ep_data.title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            PodcastDLResult::DLFileWriteError(ep_data) => {
+                self.download_tracker.decrease_one(&ep_data.url);
+                self.mount_error_popup(anyhow!("download failed for episode: {}", ep_data.title));
+                self.show_message_timeout_label_help(
+                    self.download_tracker
+                        .message_download_error_file_write(&ep_data.title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+        }
+    }
+
     /// Handle Player related messages & events
     fn update_player(&mut self, msg: PlayerMsg) -> Option<Msg> {
         match msg {
@@ -403,7 +470,7 @@ impl Model {
         None
     }
 
-    /// Switch the main view / layout
+    /// Switch the main view / layout.
     fn update_layout(&mut self, msg: MainLayoutMsg) -> Option<Msg> {
         match msg {
             MainLayoutMsg::DataBase => {
@@ -436,6 +503,8 @@ impl Model {
 
                 self.layout = TermusicLayout::DataBase;
                 self.playlist_switch_layout();
+                self.lyric_update_title();
+                self.lyric_update();
             }
             MainLayoutMsg::TreeView => {
                 let mut need_to_set_focus = true;
@@ -457,6 +526,8 @@ impl Model {
 
                 self.layout = TermusicLayout::TreeView;
                 self.playlist_switch_layout();
+                self.lyric_update_title();
+                self.lyric_update();
             }
             MainLayoutMsg::Podcast => {
                 let mut need_to_set_focus = true;
@@ -490,11 +561,15 @@ impl Model {
                 self.layout = TermusicLayout::Podcast;
                 self.podcast_sync_feeds_and_episodes();
                 self.playlist_switch_layout();
+                self.lyric_update_title();
+                self.lyric_update();
             }
         }
 
         None
     }
+
+    /// Handle all [`DBMsg`] messages. Sub-function for [`update`](Self::update).
     fn update_database_list(&mut self, msg: DBMsg) -> Option<Msg> {
         match msg {
             DBMsg::CriteriaBlurDown | DBMsg::SearchTracksBlurUp => {
@@ -519,7 +594,7 @@ impl Model {
             DBMsg::AddPlaylist(index) => {
                 if !self.dw.search_tracks.is_empty() {
                     if let Some(track) = self.dw.search_tracks.get(index) {
-                        let file = PathBuf::from(&track.file);
+                        let file = track.as_pathbuf();
                         if let Err(e) = self.playlist_add(&file) {
                             self.mount_error_popup(e.context("playlist add"));
                         }
@@ -557,6 +632,7 @@ impl Model {
         None
     }
 
+    /// Handle all [`LIMsg`] messages. Sub-function for [`update`](Self::update).
     fn update_library(&mut self, msg: LIMsg) {
         match msg {
             LIMsg::TreeBlur => {
@@ -593,6 +669,7 @@ impl Model {
         }
     }
 
+    /// Handle all [`YSMsg`] messages. Sub-function for [`update`](Self::update).
     fn update_youtube_search(&mut self, msg: YSMsg) {
         match msg {
             YSMsg::InputPopupShow => {
@@ -622,12 +699,20 @@ impl Model {
             YSMsg::TablePopupCloseCancel => {
                 self.umount_youtube_search_table_popup();
             }
-            YSMsg::TablePopupNext => {
+            YSMsg::ReqNextPage => {
                 self.youtube_options_next_page();
             }
-            YSMsg::TablePopupPrevious => {
+            YSMsg::ReqPreviousPage => {
                 self.youtube_options_prev_page();
             }
+            YSMsg::PageLoaded(data) => {
+                self.youtube_options.data = data;
+                self.sync_youtube_options();
+            }
+            YSMsg::PageLoadError(err) => {
+                self.mount_error_popup(anyhow!(err));
+            }
+
             YSMsg::TablePopupCloseOk(index) => {
                 if let Err(e) = self.youtube_options_download(index) {
                     self.library_reload_with_node_focus(None);
@@ -643,9 +728,52 @@ impl Model {
                 self.redraw = true;
                 self.mount_error_popup(anyhow!("Youtube search fail: {e}"));
             }
+            YSMsg::Download(msg) => self.update_ys_download_msg(msg),
         }
     }
 
+    /// Handle all [`YSMsg`] messages. Sub-function for [`update_youtube_search`](Self::update_youtube_search).
+    fn update_ys_download_msg(&mut self, msg: YTDLMsg) {
+        match msg {
+            YTDLMsg::Start(url, title) => {
+                self.download_tracker.increase_one(&*url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_download_start(&title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            YTDLMsg::Success(url) => {
+                self.download_tracker.decrease_one(&url);
+                self.show_message_timeout_label_help(
+                    self.download_tracker.message_download_complete(),
+                    None,
+                    None,
+                    None,
+                );
+            }
+            YTDLMsg::Completed(_url, file) => {
+                if self.download_tracker.visible() {
+                    return;
+                }
+                self.library_reload_with_node_focus(file);
+            }
+            YTDLMsg::Err(url, title, error_message) => {
+                self.download_tracker.decrease_one(&url);
+                self.mount_error_popup(anyhow!("download failed: {error_message}"));
+                self.show_message_timeout_label_help(
+                    self.download_tracker
+                        .message_download_error_response(&title),
+                    None,
+                    None,
+                    None,
+                );
+            }
+        }
+    }
+
+    /// Handle all [`GSMsg`] messages. Sub-function for [`update`](Self::update).
     #[allow(clippy::too_many_lines)]
     fn update_general_search(&mut self, msg: &GSMsg) {
         match msg {
@@ -763,12 +891,14 @@ impl Model {
             }
         }
     }
-    fn update_delete_confirmation(&mut self, msg: &Msg) -> Option<Msg> {
+
+    /// Handle all [`DeleteConfirmMsg`] messages. Sub-function for [`update`](Self::update).
+    fn update_delete_confirmation(&mut self, msg: &DeleteConfirmMsg) -> Option<Msg> {
         match msg {
-            Msg::DeleteConfirmShow => {
+            DeleteConfirmMsg::Show => {
                 self.library_before_delete();
             }
-            Msg::DeleteConfirmCloseCancel => {
+            DeleteConfirmMsg::CloseCancel => {
                 if self.app.mounted(&Id::DeleteConfirmRadioPopup) {
                     let _drop = self.app.umount(&Id::DeleteConfirmRadioPopup);
                 }
@@ -776,7 +906,7 @@ impl Model {
                     let _drop = self.app.umount(&Id::DeleteConfirmInputPopup);
                 }
             }
-            Msg::DeleteConfirmCloseOk => {
+            DeleteConfirmMsg::CloseOk => {
                 if self.app.mounted(&Id::DeleteConfirmRadioPopup) {
                     let _drop = self.app.umount(&Id::DeleteConfirmRadioPopup);
                 }
@@ -787,10 +917,11 @@ impl Model {
                     self.mount_error_popup(e.context("library delete song"));
                 }
             }
-            _ => {}
         }
         None
     }
+
+    /// Handle all [`PLMsg`] messages. Sub-function for [`update`](Self::update).
     fn update_playlist(&mut self, msg: &PLMsg) {
         match msg {
             PLMsg::Add(current_node) => {
@@ -877,14 +1008,14 @@ impl Model {
         let delay = time_out.unwrap_or(10);
 
         Handle::current().spawn(async move {
-            let _ = tx.send(Msg::Download(DLMsg::MessageShow((
+            let _ = tx.send(Msg::Notification(NotificationMsg::MessageShow((
                 title_string.clone(),
                 text_string.clone(),
             ))));
 
             sleep(Duration::from_secs(delay)).await;
 
-            let _ = tx.send(Msg::Download(DLMsg::MessageHide((
+            let _ = tx.send(Msg::Notification(NotificationMsg::MessageHide((
                 title_string,
                 text_string,
             ))));
@@ -910,82 +1041,6 @@ impl Model {
         }
     }
 
-    // update other messages
-    pub fn update_outside_msg(&mut self) {
-        if let Ok(msg) = self.rx_to_main.try_recv() {
-            self.update(Some(msg));
-        }
-    }
-
-    // change status bar text to indicate the downloading state
-    fn update_download_msg(&mut self, msg: DLMsg) -> Option<Msg> {
-        self.redraw = true;
-        match msg {
-            DLMsg::DownloadRunning(url, title) => {
-                self.download_tracker.increase_one(&*url);
-                self.show_message_timeout_label_help(
-                    self.download_tracker.message_download_start(&title),
-                    None,
-                    None,
-                    None,
-                );
-            }
-            DLMsg::DownloadSuccess(url) => {
-                self.download_tracker.decrease_one(&url);
-                self.show_message_timeout_label_help(
-                    self.download_tracker.message_download_complete(),
-                    None,
-                    None,
-                    None,
-                );
-
-                if self.app.mounted(&Id::TagEditor(IdTagEditor::LabelHint)) {
-                    self.umount_tageditor();
-                }
-            }
-            DLMsg::DownloadCompleted(_url, file) => {
-                if self.download_tracker.visible() {
-                    return None;
-                }
-                self.library_reload_with_node_focus(file);
-            }
-            DLMsg::DownloadErrDownload(url, title, error_message) => {
-                self.download_tracker.decrease_one(&url);
-                self.mount_error_popup(anyhow!("download failed: {error_message}"));
-                self.show_message_timeout_label_help(
-                    self.download_tracker
-                        .message_download_error_response(&title),
-                    None,
-                    None,
-                    None,
-                );
-            }
-            DLMsg::DownloadErrEmbedData(_url, title) => {
-                self.mount_error_popup(anyhow!("download ok but tag info is not complete."));
-                self.show_message_timeout_label_help(
-                    self.download_tracker
-                        .message_download_error_embed_data(&title),
-                    None,
-                    None,
-                    None,
-                );
-            }
-            DLMsg::MessageShow((title, text)) => {
-                self.mount_message(&title, &text);
-            }
-            DLMsg::MessageHide((title, text)) => {
-                self.umount_message(&title, &text);
-            }
-            DLMsg::FetchPhotoSuccess(image_wrapper) => {
-                self.show_image(&image_wrapper.data).ok();
-            }
-            DLMsg::FetchPhotoErr(err_text) => {
-                self.show_message_timeout_label_help(err_text, None, None, None);
-            }
-        }
-        None
-    }
-
     /// Handle & update [`SavePlaylistMsg`] related components.
     fn update_save_playlist(&mut self, msg: SavePlaylistMsg) -> Option<Msg> {
         match msg {
@@ -1021,4 +1076,127 @@ impl Model {
 
         None
     }
+
+    /// Handle all [`ServerReqResponse`].
+    fn update_server_resp_msg(&mut self, msg: ServerReqResponse) -> Option<Msg> {
+        match msg {
+            ServerReqResponse::GetProgress(response) => {
+                let pprogress: PlayerProgress = response.progress.unwrap_or_default().into();
+                self.progress_update(
+                    pprogress.position,
+                    pprogress.total_duration.unwrap_or_default(),
+                );
+
+                self.lyric_update_for_radio(response.radio_title);
+
+                self.playback
+                    .set_status(RunningStatus::from_u32(response.status));
+                // "GetProgress" is, as of ~termusic 0.11.0~0.12.0, only called initially or having missed events, so everything should be reloaded.
+                self.player_update_current_track_after();
+            }
+            ServerReqResponse::FullPlaylist(playlist_tracks) => {
+                info!("Processing Playlist from server");
+                let current_track_index = playlist_tracks.current_track_index;
+                if let Err(err) = self
+                    .playback
+                    .load_from_grpc(playlist_tracks, &self.podcast.db_podcast)
+                {
+                    self.mount_error_popup(err);
+                }
+
+                self.playlist_sync();
+
+                self.handle_current_track_index(
+                    usize::try_from(current_track_index).unwrap(),
+                    true,
+                );
+            }
+        }
+
+        None
+    }
+
+    /// Handle Stream updates [`UpdateEvents`].
+    ///
+    /// In case of lag, sends a [`TuiCmd::GetProgress`].
+    fn update_update_events_msg(&mut self, msg: UpdateEvents) -> Option<Msg> {
+        match msg {
+            UpdateEvents::MissedEvents { amount } => {
+                warn!("Stream Lagged, missed events: {amount}");
+                // we know that we missed events, force to get full information from GetProgress endpoint
+                self.command(TuiCmd::GetProgress);
+            }
+            UpdateEvents::VolumeChanged { volume } => {
+                self.config_server.write().settings.player.volume = volume;
+            }
+            UpdateEvents::SpeedChanged { speed } => {
+                self.config_server.write().settings.player.speed = speed;
+            }
+            UpdateEvents::PlayStateChanged { playing } => {
+                self.playback.set_status(RunningStatus::from_u32(playing));
+                self.progress_update_title();
+            }
+            UpdateEvents::TrackChanged(track_changed_info) => {
+                if let Some(progress) = track_changed_info.progress {
+                    self.progress_update(
+                        progress.position,
+                        progress.total_duration.unwrap_or_default(),
+                    );
+                }
+
+                if track_changed_info.current_track_updated {
+                    self.handle_current_track_index(
+                        usize::try_from(track_changed_info.current_track_index).unwrap(),
+                        false,
+                    );
+                }
+
+                if let Some(title) = track_changed_info.title {
+                    self.lyric_update_for_radio(title);
+                }
+            }
+            UpdateEvents::GaplessChanged { gapless } => {
+                self.config_server.write().settings.player.gapless = gapless;
+            }
+            UpdateEvents::Progress(progress) => {
+                self.progress_update(
+                    progress.position,
+                    progress.total_duration.unwrap_or_default(),
+                );
+            }
+            UpdateEvents::PlaylistChanged(ev) => {
+                if let Err(err) = self.update_update_events_playlist_msg(ev) {
+                    self.mount_error_popup(err);
+                }
+            }
+        }
+
+        None
+    }
+
+    /// Handle Playlist Update Events [`UpdatePlaylistEvents`].
+    fn update_update_events_playlist_msg(&mut self, msg: UpdatePlaylistEvents) -> Result<()> {
+        match msg {
+            UpdatePlaylistEvents::PlaylistAddTrack(playlist_add_track) => {
+                self.handle_playlist_add(playlist_add_track)?;
+            }
+            UpdatePlaylistEvents::PlaylistRemoveTrack(playlist_remove_track) => {
+                self.handle_playlist_remove(&playlist_remove_track)?;
+            }
+            UpdatePlaylistEvents::PlaylistCleared => {
+                self.handle_playlist_clear();
+            }
+            UpdatePlaylistEvents::PlaylistLoopMode(loop_mode) => {
+                self.handle_playlist_loopmode(&loop_mode)?;
+            }
+            UpdatePlaylistEvents::PlaylistSwapTracks(swapped_tracks) => {
+                self.handle_playlist_swap_tracks(&swapped_tracks)?;
+            }
+            UpdatePlaylistEvents::PlaylistShuffled(shuffled) => {
+                self.handle_playlist_shuffled(shuffled)?;
+            }
+        }
+
+        Ok(())
+    }
 }
