@@ -1,27 +1,23 @@
-//! SPDX-License-Identifier: MIT
-
 use std::path::{Path, PathBuf};
 use std::sync::{Arc, LazyLock};
 use std::thread;
 use std::time::Duration;
 
-use anyhow::{bail, Context, Result};
+use anyhow::{Context, Result, anyhow, bail};
 use id3::TagLike;
 use id3::Version::Id3v24;
 use regex::Regex;
 use shell_words;
-use termusiclib::ids::Id;
-use termusiclib::invidious::Instance;
+use termusiclib::invidious::{Instance, YoutubeVideo};
 use termusiclib::track::DurationFmtShort;
-use termusiclib::types::{DLMsg, Msg};
-use termusiclib::types::{YSMsg, YoutubeOptions};
 use termusiclib::utils::get_parent_folder;
-use tokio::runtime::Handle;
 use tuirealm::props::{Alignment, AttrValue, Attribute, TableBuilder, TextSpan};
 use tuirealm::{State, StateValue};
 use ytd_rs::{Arg, YoutubeDL};
 
 use super::Model;
+use crate::ui::ids::Id;
+use crate::ui::msg::{Msg, YSMsg};
 
 #[expect(dead_code)]
 static RE_FILENAME: LazyLock<Regex> =
@@ -30,6 +26,85 @@ static RE_FILENAME: LazyLock<Regex> =
 static RE_FILENAME_YTDLP: LazyLock<Regex> =
     LazyLock::new(|| Regex::new(r"\[ExtractAudio\] Destination: (?P<name>.*)\.mp3").unwrap());
 
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct YoutubeData {
+    pub items: Vec<YoutubeVideo>,
+    pub page: u32,
+}
+
+impl Default for YoutubeData {
+    fn default() -> Self {
+        Self {
+            items: Vec::new(),
+            page: 1,
+        }
+    }
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Default)]
+pub struct YoutubeOptions {
+    pub data: YoutubeData,
+    pub invidious_instance: Instance,
+}
+
+impl YoutubeOptions {
+    pub fn get_by_index(&self, index: usize) -> Result<&YoutubeVideo> {
+        if let Some(item) = self.data.items.get(index) {
+            return Ok(item);
+        }
+        Err(anyhow!("index not found"))
+    }
+
+    /// Fetch the previous page's content if there is a previous page.
+    ///
+    /// The returned Future does not need the lifetime of `self` for the fetch and is safe to [`Send`].
+    pub fn get_prev_page(&self) -> Option<impl Future<Output = Result<YoutubeData>> + use<>> {
+        if self.data.page > 1 {
+            let mut res = YoutubeData {
+                page: self.data.page - 1,
+                ..Default::default()
+            };
+            let instance = self.invidious_instance.clone();
+
+            return Some(async move {
+                res.items = instance.get_search_query(res.page).await?;
+                Ok(res)
+            });
+        }
+
+        None
+    }
+
+    /// Fetch the next page's content.
+    ///
+    /// The returned Future does not need the lifetime of `self` for the fetch and is safe to [`Send`].
+    pub fn get_next_page(&self) -> impl Future<Output = Result<YoutubeData>> + use<> {
+        let mut res = YoutubeData {
+            page: self.data.page + 1,
+            ..Default::default()
+        };
+        let instance = self.invidious_instance.clone();
+
+        async move {
+            res.items = instance.get_search_query(res.page).await?;
+            Ok(res)
+        }
+    }
+
+    #[must_use]
+    pub const fn page(&self) -> u32 {
+        self.data.page
+    }
+
+    pub fn is_empty(&self) -> bool {
+        self.data.items.is_empty()
+    }
+
+    pub fn items(&self) -> &[YoutubeVideo] {
+        &self.data.items
+    }
+}
+
 impl Model {
     pub fn youtube_options_download(&mut self, index: usize) -> Result<()> {
         // download from search result here
@@ -49,8 +124,10 @@ impl Model {
             match Instance::new(&keyword).await {
                 Ok((instance, result)) => {
                     let youtube_options = YoutubeOptions {
-                        items: result,
-                        page: 1,
+                        data: YoutubeData {
+                            items: result,
+                            page: 1,
+                        },
                         invidious_instance: instance,
                     };
                     tx.send(Msg::YoutubeSearch(YSMsg::YoutubeSearchSuccess(
@@ -67,35 +144,47 @@ impl Model {
     }
 
     /// This function requires to be run in a tokio Runtime context
-    pub fn youtube_options_prev_page(&mut self) {
-        // this needs to be wrapped as this is not running another thread but some main-runtime thread and so needs to inform the runtime to hand-off other tasks
-        // though i am not fully sure if that is 100% the case, this avoid the panic though
-        tokio::task::block_in_place(move || {
-            Handle::current().block_on(async {
-                match self.youtube_options.prev_page().await {
-                    Ok(()) => self.sync_youtube_options(),
-                    Err(e) => self.mount_error_popup(e.context("youtube-dl search")),
+    pub fn youtube_options_prev_page(&self) {
+        let tx_to_main = self.tx_to_main.clone();
+
+        let Some(fut) = self.youtube_options.get_prev_page() else {
+            return;
+        };
+
+        tokio::task::spawn(async move {
+            match fut.await {
+                Ok(data) => {
+                    let _ = tx_to_main.send(Msg::YoutubeSearch(YSMsg::PageLoaded(data)));
                 }
-            });
+                Err(err) => {
+                    let _ =
+                        tx_to_main.send(Msg::YoutubeSearch(YSMsg::PageLoadError(err.to_string())));
+                }
+            }
         });
     }
 
     /// This function requires to be run in a tokio Runtime context
     pub fn youtube_options_next_page(&mut self) {
-        // this needs to be wrapped as this is not running another thread but some main-runtime thread and so needs to inform the runtime to hand-off other tasks
-        // though i am not fully sure if that is 100% the case, this avoid the panic though
-        tokio::task::block_in_place(move || {
-            Handle::current().block_on(async {
-                match self.youtube_options.next_page().await {
-                    Ok(()) => self.sync_youtube_options(),
-                    Err(e) => self.mount_error_popup(e.context("youtube-dl search")),
+        let tx_to_main = self.tx_to_main.clone();
+
+        let fut = self.youtube_options.get_next_page();
+
+        tokio::task::spawn(async move {
+            match fut.await {
+                Ok(data) => {
+                    let _ = tx_to_main.send(Msg::YoutubeSearch(YSMsg::PageLoaded(data)));
                 }
-            });
+                Err(err) => {
+                    let _ =
+                        tx_to_main.send(Msg::YoutubeSearch(YSMsg::PageLoadError(err.to_string())));
+                }
+            }
         });
     }
 
     pub fn sync_youtube_options(&mut self) {
-        if self.youtube_options.items.is_empty() {
+        if self.youtube_options.is_empty() {
             let table = TableBuilder::default()
                 .add_col(TextSpan::from("No results."))
                 .add_col(TextSpan::from(
@@ -113,7 +202,7 @@ impl Model {
         }
 
         let mut table: TableBuilder = TableBuilder::default();
-        for (idx, record) in self.youtube_options.items.iter().enumerate() {
+        for (idx, record) in self.youtube_options.items().iter().enumerate() {
             if idx > 0 {
                 table.add_row();
             }
@@ -137,11 +226,11 @@ impl Model {
 
         if let Some(domain) = &self.youtube_options.invidious_instance.domain {
             let title = format!(
-                    "\u{2500}\u{2500}\u{2500} Page {} \u{2500}\u{2500}\u{2500}\u{2524} {} \u{251c}\u{2500}\u{2500} {} \u{2500}\u{2500}\u{2500}\u{2500}\u{2500}",
-                    self.youtube_options.page(),
-                    "Tab/Shift+Tab switch pages",
-                    domain,
-                );
+                "\u{2500}\u{2500}\u{2500} Page {} \u{2500}\u{2500}\u{2500}\u{2524} {} \u{251c}\u{2500}\u{2500} {} \u{2500}\u{2500}\u{2500}\u{2500}\u{2500}",
+                self.youtube_options.page(),
+                "Tab/Shift+Tab switch pages",
+                domain,
+            );
             self.app
                 .attr(
                     &Id::YoutubeSearchTablePopup,
@@ -187,10 +276,10 @@ impl Model {
         let url: Arc<str> = Arc::from(url);
 
         thread::spawn(move || -> Result<()> {
-            tx.send(Msg::Download(DLMsg::DownloadRunning(
+            tx.send(Msg::YoutubeSearch(YSMsg::Download(YTDLMsg::Start(
                 url.clone(),
                 "youtube music".to_string(),
-            )))
+            ))))
             .ok();
             // start download
             let download = ytd.download();
@@ -198,16 +287,18 @@ impl Model {
             // check what the result is and print out the path to the download or the error
             match download {
                 Ok(result) => {
-                    tx.send(Msg::Download(DLMsg::DownloadSuccess(url.clone())))
-                        .ok();
+                    tx.send(Msg::YoutubeSearch(YSMsg::Download(YTDLMsg::Success(
+                        url.clone(),
+                    ))))
+                    .ok();
                     // here we extract the full file name from download output
                     if let Some(file_fullname) =
                         extract_filepath(result.output(), &path.to_string_lossy())
                     {
-                        tx.send(Msg::Download(DLMsg::DownloadCompleted(
+                        tx.send(Msg::YoutubeSearch(YSMsg::Download(YTDLMsg::Completed(
                             url,
                             Some(file_fullname.clone()),
-                        )))
+                        ))))
                         .ok();
 
                         // here we remove downloaded live_chat.json file
@@ -215,19 +306,23 @@ impl Model {
 
                         embed_downloaded_lrc(&path, &file_fullname);
                     } else {
-                        tx.send(Msg::Download(DLMsg::DownloadCompleted(url, None)))
-                            .ok();
+                        tx.send(Msg::YoutubeSearch(YSMsg::Download(YTDLMsg::Completed(
+                            url, None,
+                        ))))
+                        .ok();
                     }
                 }
                 Err(e) => {
-                    tx.send(Msg::Download(DLMsg::DownloadErrDownload(
+                    tx.send(Msg::YoutubeSearch(YSMsg::Download(YTDLMsg::Err(
                         url.clone(),
                         "youtube music".to_string(),
                         e.to_string(),
-                    )))
+                    ))))
+                    .ok();
+                    tx.send(Msg::YoutubeSearch(YSMsg::Download(YTDLMsg::Completed(
+                        url, None,
+                    ))))
                     .ok();
-                    tx.send(Msg::Download(DLMsg::DownloadCompleted(url, None)))
-                        .ok();
                 }
             }
             Ok(())
@@ -236,6 +331,28 @@ impl Model {
     }
 }
 
+pub type YTDLMsgURL = Arc<str>;
+
+#[derive(Clone, PartialEq, Eq, Debug)]
+pub enum YTDLMsg {
+    /// Indicates a Start of a download.
+    ///
+    /// `(Url, Title)`
+    Start(YTDLMsgURL, String),
+    /// Indicates the Download was a Success, though termusic post-processing is not done yet.
+    ///
+    /// `(Url)`
+    Success(YTDLMsgURL),
+    /// Indicates the Download thread finished in both Success or Error.
+    ///
+    /// `(Url, Filename)`
+    Completed(YTDLMsgURL, Option<String>),
+    /// Indicates that the Download has Errored and has been aborted.
+    ///
+    /// `(Url, Title, ErrorAsString)`
+    Err(YTDLMsgURL, String, String),
+}
+
 // This just parsing the output from youtubedl to get the audio path
 // This is used because we need to get the song name
 // example ~/path/to/song/song.mp3
@@ -354,14 +471,14 @@ fn parse_args(input: &str) -> Result<Vec<ArgOrVal>, shell_words::ParseError> {
         .map(|token| {
             if token.starts_with("--") {
                 if token.contains('=') {
-                    ArgOrVal::ArgumentWithVal(token.to_string())
+                    ArgOrVal::ArgumentWithVal(token)
                 } else {
-                    ArgOrVal::Argument(token.to_string())
+                    ArgOrVal::Argument(token)
                 }
             } else if token.starts_with('-') {
-                ArgOrVal::Flag(token.to_string())
+                ArgOrVal::Flag(token)
             } else {
-                ArgOrVal::Positional(token.to_string())
+                ArgOrVal::Positional(token)
             }
         })
         .collect();
